import numpy as np


def pintar_cilindro(x_centro, y_centro, radio, altura, ax):
    angulos = np.linspace(0, 2 * np.pi, 1000)

    x = x_centro + radio * np.cos(angulos)
    y = y_centro + radio * np.sin(angulos)

    x[x < 0] = np.nan
    y[y < 0] = np.nan

    x[x > 100] = np.nan
    y[y > 100] = np.nan

    z = np.linspace(0, altura, 1000)

    x_cilindro, z_cilindro = np.meshgrid(x, z)

    ax.plot_surface(x_cilindro, y, z_cilindro)


def calcula_alturas_volumenes_circulares_objetos_relevantes(posiciones_x,
                                                            posiciones_y,
                                                            lista_objetos_relevantes,
                                                            tamanio=24,
                                                            divisiones=2,
                                                            tipo="Cilindros"):

    if len(posiciones_x) == len(posiciones_y):

        print("Lista de objetos relevantes: ", lista_objetos_relevantes)
        print("Tamanio total: ", tamanio)
        print("Divisiones: ", divisiones)

        objeto_relevante_1 = lista_objetos_relevantes[0]
        objeto_relevante_2 = lista_objetos_relevantes[1]
        objeto_relevante_3 = lista_objetos_relevantes[2]
        objeto_relevante_4 = lista_objetos_relevantes[3]

        longitud_movimientos = len(posiciones_x)
        valor_min_divivido = int(tamanio / divisiones)
        radios_figuras = list(range(valor_min_divivido, tamanio + 1, valor_min_divivido))
        cantidad_posiciones_volumen_1 = list(np.zeros(len(radios_figuras)))
        cantidad_posiciones_volumen_2 = list(np.zeros(len(radios_figuras)))
        cantidad_posiciones_volumen_3 = list(np.zeros(len(radios_figuras)))
        cantidad_posiciones_volumen_4 = list(np.zeros(len(radios_figuras)))

        for i in range(0, longitud_movimientos):
            x_pos = posiciones_x[i]
            y_pos = posiciones_y[i]
            print("------------------------------------------------------------------------------")
            dist_1 = calcula_distancia_entre_puntos(objeto_relevante_1[0], x_pos, objeto_relevante_1[1],
                                                    y_pos)
            dist_2 = calcula_distancia_entre_puntos(objeto_relevante_2[0], x_pos, objeto_relevante_2[1],
                                                    y_pos)
            dist_3 = calcula_distancia_entre_puntos(objeto_relevante_3[0], x_pos, objeto_relevante_3[1],
                                                    y_pos)
            dist_4 = calcula_distancia_entre_puntos(objeto_relevante_4[0], x_pos, objeto_relevante_4[1],
                                                    y_pos)

            print(dist_1,"<",tamanio,": ", dist_1 < tamanio)
            if dist_1 < tamanio:
                print("Cilindros: caso 1")
                cantidad_posiciones_volumen_1 = \
                    modificar_lista_para_cilindros(cantidad_posiciones_volumen_1,
                                                   radios_figuras,
                                                   dist_1,
                                                   tipo=tipo)
            print(dist_2,"<",tamanio,": ", dist_2 < tamanio)
            if dist_2 < tamanio:
                print("Cilindros: caso 2")
                cantidad_posiciones_volumen_2 = \
                    modificar_lista_para_cilindros(cantidad_posiciones_volumen_2,
                                                   radios_figuras,
                                                   dist_2,
                                                   tipo=tipo)
            print(dist_3, "<", tamanio, ": ", dist_3 < tamanio)
            if dist_3 < tamanio:
                print("Cilindros: caso 3")
                cantidad_posiciones_volumen_3 = \
                    modificar_lista_para_cilindros(cantidad_posiciones_volumen_3,
                                                   radios_figuras,
                                                   dist_3,
                                                   tipo=tipo)
            print(dist_4, "<", tamanio, ": ", dist_4 < tamanio)
            if dist_4 < tamanio:
                print("Cilindros: caso 4")
                cantidad_posiciones_volumen_4 = \
                    modificar_lista_para_cilindros(cantidad_posiciones_volumen_4,
                                                   radios_figuras,
                                                   dist_4,
                                                   tipo=tipo)
            print("------------------------------------------------------------------------------")

        print("Cantidad de posiciones para volumen 1:")
        print(cantidad_posiciones_volumen_1)
        print("Cantidad de posiciones para volumen 2:")
        print(cantidad_posiciones_volumen_2)
        print("Cantidad de posiciones para volumen 3:")
        print(cantidad_posiciones_volumen_3)
        print("Cantidad de posiciones para volumen 4:")
        print(cantidad_posiciones_volumen_4)

        lista_cantidad_posiciones_volumenes = [cantidad_posiciones_volumen_1, cantidad_posiciones_volumen_2,
                                               cantidad_posiciones_volumen_3, cantidad_posiciones_volumen_4]

        return lista_cantidad_posiciones_volumenes, radios_figuras


def calcula_distancia_entre_puntos(x_1, x_2, y_1, y_2):
    distancia = np.sqrt((x_2 - x_1) ** 2 + (y_2 - y_1) ** 2)
    print("Distancia: ", distancia)
    return distancia


def modificar_lista_para_cilindros(lista_a_modificar, lista_radios_cilindros, distancia, tipo="cilindros"):
    if tipo == "Cilindros":
        for i in range(0, len(lista_radios_cilindros)):
            if distancia < lista_radios_cilindros[i]:
                lista_a_modificar[i] += 1
    elif tipo == "Arandelas":
        for i in range(0, len(lista_radios_cilindros)):
            if distancia < lista_radios_cilindros[i]:
                lista_a_modificar[i] += 1
                break
    return lista_a_modificar


import numpy as np


def formar_sucesiones_vectoriales(datos, tamanio_sucesiones_vectoriales):
    lista_sucesiones_vectoriales = []
    tamanio_datos = len(datos)
    print("Tamanio datos:", tamanio_datos)
    total_vectores = tamanio_datos - tamanio_sucesiones_vectoriales + 1
    print("Total vectores:", total_vectores)
    for i in range(0, total_vectores):
        vector = [dato for dato in datos[i:i+tamanio_sucesiones_vectoriales]]
        print("Vector", i)
        print(vector)
        lista_sucesiones_vectoriales.append(vector)
    return lista_sucesiones_vectoriales


def distancia_vectores_max_dif(vector_1, vector_2):
    if len(vector_1) == len(vector_2):
        print("Realizo la distancia vectorial")
        longitud_vectores = len(vector_1)
        distancias_totales = []
        for i in range(0, longitud_vectores):
            pos_v1 = vector_1[i]
            pos_v2 = vector_2[i]
            distancia_actual = abs(pos_v1 - pos_v2)
            distancias_totales.append(distancia_actual)
        return max(distancias_totales)
    else:
        print("No se puede realizar la distancia vectorial.")
        return None


def compara_val_dist_con_umbral(distancia, umbral):
    if distancia < umbral:
        return 1
    else:
        return 0


def obtener_coeficiente_c_i(lista_vectores, posicion, umbral):
    vector_a_comparar = lista_vectores[posicion]
    cantidad_vectores = len(lista_vectores)
    cantidad_vectores_mayores_umbral = 0
    for vector in lista_vectores:
        distancia_resultante = distancia_vectores_max_dif(vector_a_comparar, vector)
        cantidad_vectores_mayores_umbral += compara_val_dist_con_umbral(distancia=distancia_resultante,
                                                                        umbral=umbral)
    print("Cantidad vectores mayores a umbral: ", cantidad_vectores_mayores_umbral)
    print("longitud_vectores")
    resultado = cantidad_vectores_mayores_umbral/cantidad_vectores
    return resultado


def obtener_todos_coeficientes_c(lista_vectores, umbral):
    lista_coeficientes_c = []
    for i in range(0, len(lista_vectores)):
        c_actual = obtener_coeficiente_c_i(lista_vectores=lista_vectores,
                                           posicion=i,
                                           umbral=umbral)
        lista_coeficientes_c.append(c_actual)
    return lista_coeficientes_c


def phi(coeficientes_c):
    return sum(np.log(coeficientes_c))/len(coeficientes_c)


def entropia_aprox(lista_datos, tamanio_vectorial, umbral):
    lista_vectores = formar_sucesiones_vectoriales(datos=lista_datos,
                                                   tamanio_sucesiones_vectoriales=tamanio_vectorial)
    coef_c_i_m = obtener_todos_coeficientes_c(lista_vectores, umbral)
    print("Coeficientes_m", coef_c_i_m)
    phi_m = phi(coeficientes_c=coef_c_i_m)
    lista_vectores = formar_sucesiones_vectoriales(datos=lista_datos,
                                                   tamanio_sucesiones_vectoriales=tamanio_vectorial+1)
    coef_c_i_m = obtener_todos_coeficientes_c(lista_vectores, umbral)
    print("Coeficientes_m_1", coef_c_i_m)
    phi_m_1 = phi(coeficientes_c=coef_c_i_m)
    resultado_entropia_aprox = phi_m - phi_m_1
    return resultado_entropia_aprox







import numpy as np
import scipy.stats as st
import motus.entropia_divergencia.entropia_aprox_agil as m_eaa
import motus.entropia_divergencia.entropia_aprox_agil_generalizada as m_eaag


def agrega_categoria_entropia(diccionario_entropia, categoria, datos):
    print("Agrego categoria de entropia: ")
    print("--------------------------------------------------------------")
    print("Diccionario:", diccionario_entropia)
    print("Categoria", categoria)
    print("Datos:", datos)
    nuevos_datos = np.array(datos)
    if categoria not in diccionario_entropia:
        diccionario_entropia[categoria] = st.entropy(nuevos_datos)
    print("Nuevo diccionario: ", diccionario_entropia)


def agrega_categoria_entropia_aproximada(diccionario_entropia,
                                         categoria,
                                         datos,
                                         tamanio_vectorial,
                                         umbral,
                                         sobreposicion):
    print("Agrego categoria de entropia aproximada: ")
    print("--------------------------------------------------------------")
    print("Diccionario:", diccionario_entropia)
    print("Categoria", categoria)
    print("Datos:", datos)
    nuevos_datos = np.array(datos)
    if categoria not in diccionario_entropia:
        print("Procesando informacion")
        diccionario_entropia[categoria] = \
            m_eaag.calcula_entropia_aprox_agil_general(datos=nuevos_datos,
                                                       tamanio_vector=tamanio_vectorial,
                                                       umbral=umbral,
                                                       sobreposicion=sobreposicion)
    print("Nuevo diccionario: ", diccionario_entropia)


def modificar_arreglo_para_evitar_divergencia_infinita(datos):
    nuevos_datos = np.copy(datos)
    indices_con_valores_0 = np.argwhere(nuevos_datos == 0)

    for indice in indices_con_valores_0:
        nuevos_datos[indice] = 1

    return nuevos_datos


def calcula_divergencia_de_2_arreglos(pk, qk):
    nuevo_qk = modificar_arreglo_para_evitar_divergencia_infinita(qk)
    divergencia = st.entropy(pk, nuevo_qk)
    return divergencia


def calcula_divergencias_de_lista_arreglos(lista_arreglos):

    resultados_divergencia = []

    for i in range(0, len(lista_arreglos) - 1):
        pk = lista_arreglos[i]
        qk = lista_arreglos[i+1]

        divergencia_pk_qk = calcula_divergencia_de_2_arreglos(pk, qk)

        resultados_divergencia.append(divergencia_pk_qk)

    return resultados_divergencia



import numpy as np


def calcula_distancias_a_objetos(posiciones_x, posiciones_y, lista_objetos):

    objeto_1_x = lista_objetos[0][0]
    objeto_1_y = lista_objetos[0][1]

    objeto_2_x = lista_objetos[1][0]
    objeto_2_y = lista_objetos[1][1]

    objeto_3_x = lista_objetos[2][0]
    objeto_3_y = lista_objetos[2][1]

    objeto_4_x = lista_objetos[3][0]
    objeto_4_y = lista_objetos[3][1]

    dist_objeto_1 = ((objeto_1_x - posiciones_x) ** 2 + (objeto_1_y - posiciones_y) ** 2) ** 0.5
    dist_objeto_2 = ((objeto_2_x - posiciones_x) ** 2 + (objeto_2_y - posiciones_y) ** 2) ** 0.5
    dist_objeto_3 = ((objeto_3_x - posiciones_x) ** 2 + (objeto_3_y - posiciones_y) ** 2) ** 0.5
    dist_objeto_4 = ((objeto_4_x - posiciones_x) ** 2 + (objeto_4_y - posiciones_y) ** 2) ** 0.5

    return dist_objeto_1, dist_objeto_2, dist_objeto_3, dist_objeto_4


def resultados_distancias(posiciones_x, posiciones_y, lista_distancias):

    dist_1 = lista_distancias[0]
    dist_2 = lista_distancias[1]
    dist_3 = lista_distancias[2]
    dist_4 = lista_distancias[3]

    distancia_minima_objetos_2_3_4 = np.minimum(dist_2,
                                                np.minimum(dist_3, dist_4))
    distancia_minima_todos_objetos = np.minimum(dist_1, distancia_minima_objetos_2_3_4)

    longitud_distancias = len(dist_1)

    nuevas_posiciones_x = posiciones_x.fillna(0)
    nuevas_posiciones_y = posiciones_y.fillna(0)

    distancia_total_recorrida = 0

    for i in range(1, longitud_distancias):
        distancia_x = (nuevas_posiciones_x.iloc[i] - nuevas_posiciones_x.iloc[i - 1]) ** 2
        distancia_y = (nuevas_posiciones_y.iloc[i] - nuevas_posiciones_y.iloc[i - 1]) ** 2
        distancia_total_recorrida += (distancia_x + distancia_y)**(1/2)

    return distancia_minima_objetos_2_3_4, distancia_minima_todos_objetos, distancia_total_recorrida


def obtener_arreglo_con_valores_promediados_para_distancia_a_objeto(arreglo, tamanio_intervalo):
    arreglo_con_promedio = []
    for i in range(len(arreglo)):
        valor_promediado = obtener_valor_promediado_para_distancia_a_objeto(arreglo, i, tamanio_intervalo)
        arreglo_con_promedio.append(valor_promediado)
    return arreglo_con_promedio


def obtener_valor_promediado_para_distancia_a_objeto(arreglo, indice, tamanio_intervalo):
    if indice - tamanio_intervalo < 0:
        left = arreglo[0:indice]
    else:
        left = arreglo[(indice - tamanio_intervalo):indice]
    if indice + tamanio_intervalo > len(arreglo):
        right = arreglo[(indice + 1):len(arreglo)]
    else:
        right = arreglo[indice + 1:(indice + tamanio_intervalo) + 1]
    arreglo_final = left + [arreglo[indice]] + right
    promedio = sum(arreglo_final) / len(arreglo_final)
    return promedio





import numpy as np
from motus.entropia_divergencia.entropia_aprox import phi


def calcular_probabilidad_vector_unico(datos, vector, umbral):
    probabilidad_actual = 0
    tamanio_vector = len(vector)
    for i in range(0, len(datos) - tamanio_vector + 1):
        print("-------------------------------")
        print("Vector", i)
        print(vector)
        print("[" + str(datos[i]) + ",", str(datos[i + 1]) + "]")
        abs_1 = np.abs(datos[i] - vector[0])
        abs_2 = np.abs(datos[i + 1] - vector[1])
        dist = max(abs_1, abs_2)
        print("Dist:", dist)
        if dist <= umbral:
            probabilidad_actual += 1
    probabilidad_actual /= (len(datos) - tamanio_vector + 1)
    return probabilidad_actual


def calcular_probabilidad_vector(datos, vector, umbral):
    probabilidad_actual = 0
    probabilidad_actual_extra = 0
    tamanio_vector = len(vector)
    for i in range(0, len(datos) - tamanio_vector + 1):
        print("-------------------------------")
        print("Vector", i)
        print(vector)
        print("["+str(datos[i])+",", str(datos[i+1])+",", str(datos[i+2])+"]")
        abs_1 = np.abs(datos[i] - vector[0])
        abs_2 = np.abs(datos[i+1] - vector[1])
        abs_3 = np.abs(datos[i+2] - vector[2])
        dist = max(abs_1, abs_2)
        dist_extra = max(dist, abs_3)
        print("Dist:", dist)
        print("Dist extra:", dist_extra)
        if dist <= umbral:
            probabilidad_actual += 1
        if dist_extra <= umbral:
            probabilidad_actual_extra += 1
    print("Vector",tamanio_vector)
    print(vector[0:2])
    print(datos[-2:])
    abs_1 = np.abs(datos[-2] - vector[0])
    abs_2 = np.abs(datos[-1] - vector[1])
    dist = max(abs_1, abs_2)
    if dist <= umbral:
        probabilidad_actual += 1
    probabilidad_actual /= len(datos) - tamanio_vector + 2
    probabilidad_actual_extra /= len(datos) - tamanio_vector + 1
    return probabilidad_actual, probabilidad_actual_extra


def calcular_probabilidades_agil_corregido(datos, umbral):
    probabilidades_m = []
    probabilidades_m_extra = []
    for i in range(0, len(datos) - 3 + 1):
        vector_actual = datos[i:i+3]
        probabilidad_actual, probabilidad_extra = calcular_probabilidad_vector(datos, vector_actual, umbral)
        probabilidades_m.append(probabilidad_actual)
        probabilidades_m_extra.append(probabilidad_extra)
    ultimo_vector = datos[-2:]
    ultima_probabilidad = calcular_probabilidad_vector_unico(datos, ultimo_vector, umbral)
    probabilidades_m.append(ultima_probabilidad)
    return probabilidades_m, probabilidades_m_extra


def calcula_entropia_aprox_agil(datos, umbral=3):
    c1, c2 = calcular_probabilidades_agil_corregido(datos=datos, umbral=umbral)
    print("Probabilidad 1:", c1)
    print("Probabilidad 2:", c2)
    return phi(c1) - phi(c2)



import numpy as np
import pandas as pd


def determinar_rectangulo_por_base_altura_centro(base, altura, centro):
    mitad_base = base/2
    mitad_altura = altura/2

    coordenada_x = centro[0]
    coordenada_y = centro[1]

    x1 = coordenada_x - mitad_base
    x2 = coordenada_x + mitad_base
    y1 = coordenada_y - mitad_altura
    y2 = coordenada_y + mitad_altura

    return [x1, x2], [y1, y2]


def determinar_regiones_importantes_por_cuadrado(x, y, dimension_caja, numero_filas, numero_columnas):

    tamanio_horizontal = dimension_caja[0]/numero_columnas
    tamanio_vertical = dimension_caja[1]/numero_filas

    print("Tamanio horizontal: "+str(dimension_caja[0])+"/"+str(numero_columnas)+"=", str(tamanio_horizontal))
    print("Tamanio vertical: "+str(dimension_caja[1])+"/"+str(numero_filas)+"=", str(tamanio_vertical))

    x1 = x[0]
    x2 = x[1]
    y1 = y[0]
    y2 = y[1]

    limite_inf_x = obtener_limite_sobre_eje_verificando_modulo(x1, tamanio_horizontal)
    limite_sup_x = obtener_limite_sobre_eje_verificando_modulo(x2, tamanio_horizontal)
    limite_inf_y = obtener_limite_sobre_eje_verificando_modulo(y1, tamanio_horizontal)
    limite_sup_y = obtener_limite_sobre_eje_verificando_modulo(y2, tamanio_horizontal)

    print("Limite inferior x: "+str(x1)+"//"+str(tamanio_horizontal)+" = "+str(limite_inf_x))
    print("Limite superior x: "+str(x2)+"//"+str(tamanio_horizontal)+" = "+str(limite_sup_x))
    print("Limite inferior y: "+str(y1)+"//"+str(tamanio_vertical)+" = "+str(limite_inf_y))
    print("Limite superior y: "+str(y2)+"//"+str(tamanio_vertical)+" = "+str(limite_sup_y))

    lista_regiones_importantes = []

    print(range(limite_inf_x, limite_sup_x))
    print(range(limite_inf_y, limite_sup_y))
    for i in range(limite_inf_y, limite_sup_y):
        for j in range(limite_inf_x, limite_sup_x):
            lista_regiones_importantes.append(numero_columnas * i + j)

    return lista_regiones_importantes


def obtener_limite_sobre_eje_verificando_modulo(limite, tamanio_region):
    if limite%tamanio_region == 0:
        return int(limite//tamanio_region)
    else:
        return int(limite//tamanio_region)

"""
def arreglos_a_modificar(m1, m2):
    nuevo_m1 = np.copy(m1)
    nuevo_m2 = np.copy(m2)

    indices_con_valores_0 = np.argwhere(nuevo_m2 == 0)

    for indices in indices_con_valores_0:
        nuevo_m1[indices[0], indices[1]] = 0
        nuevo_m2[indices[0], indices[1]] = 1

    return nuevo_m1, nuevo_m2
"""

def genera_serie_de_datos(datos):
    return pd.Series(datos)


def genera_dataframes_de_datos(datos):
    diccionario_csv = {
        'valores': datos
    }
    tabla_csv = pd.DataFrame(diccionario_csv)
    return tabla_csv


def exporta_archivo(archivo, nombre_a_guardar, tipo):
    if tipo is "csv":
        archivo.to_csv(nombre_a_guardar)



import numpy as np


def calcula_regiones_cuadriculadas(tiempos,
                                   serie_posiciones_x, serie_posiciones_y,
                                   cantidad_filas, cantidad_columnas,
                                   dimension_caja):
    print("Calculando regiones en la caja...")

    posiciones_x = serie_posiciones_x
    posiciones_y = serie_posiciones_y

    cantidad_tiempos = len(tiempos)

    matriz_cuadriculada = np.zeros((cantidad_filas, cantidad_columnas))

    tamanio_horizontal = dimension_caja[0] / cantidad_columnas
    tamanio_vertical = dimension_caja[1] / cantidad_filas

    print("Tamanio de los cuadros: ", tamanio_horizontal, "x", tamanio_vertical)

    arreglo_region_cuadriculada = np.zeros(cantidad_tiempos)

    for k in range(cantidad_tiempos):
        # print("Posicion de k:", k)

        if not (np.isnan(posiciones_x.iloc[k])) or not (np.isnan(posiciones_y.iloc[k])):

            posicion_x = posiciones_x.iloc[k]
            posicion_y = posiciones_y.iloc[k]
            # print("Indice:", k)
            if posicion_x == dimension_caja[0]:
                # print(cantidad_columnas)
                j = cantidad_columnas - 1
                # print("Entro en opcion 1-x")
            else:
                # print("Entro en opcion 2-x")
                j = int(posiciones_x.iloc[k] // tamanio_horizontal)
            if posicion_y == dimension_caja[1]:
                # print(cantidad_filas)
                i = cantidad_filas - 1
                # print("Entro en opcion 1-y")
            else:
                i = int(posiciones_y.iloc[k] // tamanio_vertical)
                # print("Entro en opcion 2-y")
            # print("i: ", i)
            # print("j: ", j)
            matriz_cuadriculada[i][j] = matriz_cuadriculada[i][j] + 1
            arreglo_region_cuadriculada[k] = cantidad_columnas * i + j

        matriz_cuadriculada = np.flipud(matriz_cuadriculada)

    return arreglo_region_cuadriculada, matriz_cuadriculada


def obtener_indice_regiones_objeto(dimension_caja,
                                   cantidad_filas, cantidad_columnas,
                                   lista_objetos_relevantes):

    indice_region_objeto_1 = None
    indice_region_objeto_2 = None
    indice_region_objeto_3 = None
    indice_region_objeto_4 = None

    tamanio_horizontal = dimension_caja[0] / cantidad_columnas
    tamanio_vertical = dimension_caja[1] / cantidad_filas

    for objeto in lista_objetos_relevantes:

        posicion_x = objeto[0]
        posicion_y = objeto[1]

        if posicion_x == dimension_caja[0]:
            j = cantidad_columnas - 1
        else:
            j = posicion_x // tamanio_horizontal
        if posicion_y == dimension_caja[1]:
            i = cantidad_filas - 1
        else:
            i = posicion_y // tamanio_vertical

        if indice_region_objeto_1 is None:
            indice_region_objeto_1 = cantidad_columnas * i + j
        elif indice_region_objeto_2 is None:
            indice_region_objeto_2 = cantidad_columnas * i + j
        elif indice_region_objeto_3 is None:
            indice_region_objeto_3 = cantidad_columnas * i + j
        elif indice_region_objeto_4 is None:
            indice_region_objeto_4 = cantidad_columnas * i + j

    return indice_region_objeto_1, indice_region_objeto_2, indice_region_objeto_3, indice_region_objeto_4


from motus.experimento import Experimento
import motus.entropia_divergencia.entropia_divergencia as m_ed
import motus.regresion_lineal.distancias_pendientes as m_dp
from matplotlib.figure import Figure


class MultiplesExperimentos:

    def __init__(self,
                 lista_archivos,
                 dimension_caja=None,
                 cantidad_filas=10,
                 cantidad_columnas=10,
                 procesar_tiempo_archivo_irregular=False,
                 puntos_importantes=None,
                 objeto_a_promediar=1,
                 tamanio_intervalo_para_promedio=100,
                 epsilon_distancias_pendientes=1):
        self.lista_archivos = lista_archivos
        self.lista_experimentos = list()
        self.resultados_entropia = dict()
        self.resultados_divergencia = dict()
        self.pendientes_regresion_lineal = dict()
        self.resultados_pendientes_regresion_lineal = dict()

        if dimension_caja is None:
            self.dimension_caja = [100, 100]

        self.dimension_caja = dimension_caja
        self.cantidad_filas = cantidad_filas
        self.cantidad_columnas = cantidad_columnas
        self.procesar_tiempo_archivo_irregular = procesar_tiempo_archivo_irregular
        self.puntos_importantes = puntos_importantes
        self.objeto_a_promediar = objeto_a_promediar
        self.tamanio_intervalo_para_promedio = tamanio_intervalo_para_promedio
        self.epsilon_distancias_pendientes = epsilon_distancias_pendientes

    def inicializa_experimentos(self):

        print("Procesar tiempo irregular: ", self.procesar_tiempo_archivo_irregular)
        if isinstance(self.lista_archivos, dict):

            count = 0

            print("Diccionario de archivos: ", self.lista_archivos)

            for (nombre, valor) in self.lista_archivos.items():
                if valor.get() is True:
                    count += 1
                    print("conteo de experimentos: ", count)
                    nuevo_experimento = Experimento(str(nombre),
                                                    dimension_caja=self.dimension_caja,
                                                    cantidad_filas=self.cantidad_filas,
                                                    cantidad_columnas=self.cantidad_columnas,
                                                    procesar_tiempo_archivo_irregular=
                                                    self.procesar_tiempo_archivo_irregular,
                                                    puntos_importantes=self.puntos_importantes)

                    self.lista_experimentos.append(nuevo_experimento)
        else:
            for archivo in self.lista_archivos:
                nuevo_experimento = Experimento(archivo,
                                                dimension_caja=self.dimension_caja,
                                                cantidad_filas=self.cantidad_filas,
                                                cantidad_columnas=self.cantidad_columnas,
                                                procesar_tiempo_archivo_irregular=
                                                self.procesar_tiempo_archivo_irregular,
                                                )
                self.lista_experimentos.append(nuevo_experimento)

    def calcula_velocidades_multiples_experimentos(self):
        for experimento in self.lista_experimentos:
            experimento.calcula_velocidades()

    def calcula_regiones_cuadriculadas(self, arreglo_regiones_preferidas):
        for experimento in self.lista_experimentos:
            experimento.calcula_regiones_cuadriculadas()
            experimento.calcula_matrices_solo_regiones_preferidas(regiones_preferidas=arreglo_regiones_preferidas)

    def calcula_entropias_divergencias(self,
                                       tamanio_vectores=2,
                                       umbral=3,
                                       sobreposicion=True):
        for experimento in self.lista_experimentos:
            experimento.calcula_entropias(tamanio_vectores, umbral, sobreposicion)

    def calcula_regresiones_lineales(self, brinco_temporal, longitud_ventana_tiempo):
        for experimento in self.lista_experimentos:
            for tipo_arreglo in ["Velocidad", "Aceleracion"]:
                arreglo_seleccionado = experimento.regresa_tipo_arreglo(tipo_arreglo)

                procesador = experimento.procesador_regresion_lineal
                procesador.arreglo_a_procesar = arreglo_seleccionado
                procesador.brinco_temporal = brinco_temporal
                procesador.longitud_ventana_tiempo = longitud_ventana_tiempo

                procesador.realiza_procesamiento()
                procesador.agrega_categoria_regresion_lineal(experimento.regresiones_lineales,
                                                             tipo_arreglo)

    def procesa_pendientes(self):
        experimento_inicial = self.lista_experimentos[0]

        for tipo_regresion_lineal in sorted(experimento_inicial.regresiones_lineales):
            lista_pendientes = []
            for experimento in self.lista_experimentos:
                diccionario_regresion_lineal_experimento = experimento.regresiones_lineales[tipo_regresion_lineal]
                pendiente_experimento = diccionario_regresion_lineal_experimento["pendientes"]
                lista_pendientes.append(pendiente_experimento)
            self.pendientes_regresion_lineal[tipo_regresion_lineal] = lista_pendientes

    def procesa_distancias_pendientes(self, epsilon):
        print("-------------------------------------------------------------------------------")
        for tipo_regresion_lineal in sorted(self.pendientes_regresion_lineal):
            print("Regresion lineal a procesar distancias pendientes: ", tipo_regresion_lineal)
            lista_pendientes = self.pendientes_regresion_lineal[tipo_regresion_lineal]
            print("Lista de pendientes para", tipo_regresion_lineal, ": ", lista_pendientes)
            resultados_pendientes_funcion_signo = m_dp.procesa_lista_pendientes_funcion_signo(lista_pendientes,
                                                                                              epsilon)
            resultados_distancias = m_dp.calcula_distancias_para_lista_pendientes(resultados_pendientes_funcion_signo)
            self.resultados_pendientes_regresion_lineal[tipo_regresion_lineal] = resultados_distancias

    def procesa_diccionario_entropias(self):
        """
        Se lee uno de los diccionarios de entropia para conocer sus llaves. Por cada llave, se obtienen todos
        los datos correspondientes a esa entropia para cada experimento. La lista resultante se guarda en un
        diccionario, donde la llave es el tipo de entropia y el registro es la lista con los valores encontrados.

        El resultado es un diccionario con todas las entropias registradas, y listas con los valores de
        cada experimento
        """

        experimento_inicial = self.lista_experimentos[0]

        for tipo_entropia in sorted(experimento_inicial.entropias):
            valores_entropia = []
            for experimento in self.lista_experimentos:
                valor_entropia_experimento = experimento.entropias[tipo_entropia]
                valores_entropia.append(valor_entropia_experimento)
            self.resultados_entropia[tipo_entropia] = valores_entropia

    def procesa_diccionario_divergencias(self):
        lista_arreglos_regiones = list()
        lista_arreglos_solo_regiones_preferidas = list()
        lista_arreglos_solo_regiones_no_preferidas = list()
        for experimento in self.lista_experimentos:
            lista_arreglos_regiones.append(
                experimento.matriz_cuadriculada.flatten())
            lista_arreglos_solo_regiones_preferidas.append(
                experimento.matriz_cuadriculada_solo_regiones_preferidas.flatten())
            lista_arreglos_solo_regiones_no_preferidas.append(
                experimento.matriz_cuadriculada_solo_regiones_no_preferidas.flatten())
        self.calcula_divergencia("visitas_regiones", lista_arreglos_regiones)
        self.calcula_divergencia("visitas_regiones_preferidas", lista_arreglos_solo_regiones_preferidas)
        self.calcula_divergencia("visitas_regiones_no_preferidas", lista_arreglos_solo_regiones_no_preferidas)

    def calcula_divergencia(self, tipo_divergencia, lista_datos):
        lista_divergencia = m_ed.calcula_divergencias_de_lista_arreglos(lista_datos)
        self.resultados_divergencia[tipo_divergencia] = lista_divergencia

    def regresa_imagen_grafica(self, grafica):
        print("-----------------------------------------------")
        print("Grafica deseada:", grafica)

        fig = Figure(figsize=(6, 6))
        fig_plot = fig.add_subplot(111)
        fig_plot.grid(True)

        cantidad_experimentos = len(self.lista_experimentos)

        if grafica is 'entropia_visitas_regiones':
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones"])

        if grafica is 'entropia_aprox_visitas_regiones':
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones_aprox"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones_aprox"])

        elif grafica is 'entropia_visitas_regiones_preferidas':
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones_preferidas"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones_preferidas"])

        elif grafica is 'entropia_visitas_regiones_no_preferidas':
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones_no_preferidas"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["visitas_regiones_no_preferidas"])

        elif grafica is 'entropia_hist_velocidad':
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["histograma_velocidad"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos+1),
                          self.resultados_entropia["histograma_velocidad"])

        elif grafica is 'divergencia_regiones':
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_divergencia["visitas_regiones"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_divergencia["visitas_regiones"])

        elif grafica is 'divergencia_regiones_preferidas':
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_divergencia["visitas_regiones_preferidas"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_divergencia["visitas_regiones_preferidas"])

        elif grafica is 'divergencia_regiones_no_preferidas':
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_divergencia["visitas_regiones_no_preferidas"], 'o')
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_divergencia["visitas_regiones_no_preferidas"])

        elif grafica is 'pendientes_distancias_velocidad':
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_pendientes_regresion_lineal['Velocidad'], 'o')
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_pendientes_regresion_lineal['Velocidad'])

        elif grafica is 'pendientes_distancias_aceleracion':
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_pendientes_regresion_lineal['Aceleracion'], 'o')
            fig_plot.plot(range(1, cantidad_experimentos),
                          self.resultados_pendientes_regresion_lineal['Aceleracion'])

        return fig
